@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: codetable.c,v 1.1.1.1 2002/04/06 02:23:31 famao Exp $ */
+/* $Id: codetable.c,v 1.1.1.1.2.1 2002/04/12 22:22:18 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -65,21 +65,21 @@ typedef struct _CodeTable
 
 static CodeTable default_codetable[] =
 {
-	{ 0xa1a1, 2},	/* Symbol */
-	{ 0xa3a1, 1},	/* JISX0208_LATIN */
-	{ 0xa4a1, 1}, 	/* Hiragana */
-	{ 0xa5a1, 1},	/* Katakana */
-	{ 0xa6a1, 1},	/* Greek */
-	{ 0xa7a1, 1},	/* Cyrillic */
-	{ 0xa8a1, 1},	/* */
-	{ 0xb0a1, 31},	/* Kanji (JIS Level 1) */
-	{ 0xe1a1, 36},	/* Kanji (JIS Level 2) */
-	{ 0x8ea1, 1},	/* Single byte Katakana */
-	{ 0x8fa2a1, 1},	/* */
-	{ 0x8fa6a1, 1},	/* */
-	{ 0x8fa7a1, 1},	/* */
-	{ 0x8fa9a1, 3},	/* */
-	{ 0x8fb0a1, 61}, /* Supplementary Kanji */
+	{ 0xa180, 2},	/* Symbol */
+	{ 0xa380, 1},	/* JISX0208_LATIN */
+	{ 0xa480, 1}, 	/* Hiragana */
+	{ 0xa580, 1},	/* Katakana */
+	{ 0xa680, 1},	/* Greek */
+	{ 0xa780, 1},	/* Cyrillic */
+	{ 0xa880, 1},	/* */
+	{ 0xb080, 31},	/* Kanji (JIS Level 1) */
+	{ 0xe180, 36},	/* Kanji (JIS Level 2) */
+	{ 0x8e80, 1},	/* Single byte Katakana */
+	{ 0x8fa280, 1},	/* */
+	{ 0x8fa680, 1},	/* */
+	{ 0x8fa780, 1},	/* */
+	{ 0x8fa980, 3},	/* */
+	{ 0x8fb080, 61}, /* Supplementary Kanji */
 };
 
 static void for_each (GtkWidget *widget, gpointer user_data);
