@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkbuffer.c,v 1.13 2002/04/11 00:02:36 famao Exp $ */
+/* $Id: skkbuffer.c,v 1.14.2.4 2002/04/24 00:27:55 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -45,9 +45,9 @@
 #include "skkconv_kana.h"
 #include "skkconv_num.h"
 #include "skkconv_jisx0201_kana.h"
-#include "skkdict.h"
 #include "skkconv_jisx0208_latin.h"
 #include "skkutils.h"
+#include "skkdictitem.h"
 #include "skkdebug.h"
 
 /* buffer operation */
@@ -133,9 +133,11 @@ lookup_emit (SkkBuffer *buf)
 	restore = buf->candidate_count;
 	for (i = 0; i < 7; i++) {
 		candidate[i] = skk_buffer_get_cur_candidate (buf);
+		if (candidate[i]) {
+			count++;
+		}
 		if (skk_buffer_has_next_candidate (buf)) {
 			buf->candidate_count++;
-			count++;
 		} else {
 			break;
 		}
@@ -333,34 +335,61 @@ query_num (SkkBuffer *buf)
 	GList *num = NULL;
 	GList *tmp = NULL;
 	gchar *candidate;
-	gint count;
 	gint list_num;
 	gint type;
 	gchar *ret;
 
+	if (!buf->candidate_list)
+		return;
 	candidate = skkbuf_cur_candidate (buf);
+	if (!candidate)
+		return;
 	ret = skk_num_translate (candidate, buf->number_list, buf->has_num, &type);
 	if (ret) {
 		if (type == 4) {
-			num = skk_dict_query_string (ret, skkbuf_b (buf), &list_num);
+			/* in the case of #4, retry */
+			/* reset query item */
+			skk_query_set_first (skkbuf_query (buf));
+			/* query to dict */
+			num = skk_query_do_query (buf->query, ret, skkbuf_b (buf), &list_num);
+			if (!num) {
+				while (skk_query_set_next (buf->query)) {
+					num = skk_query_do_query (buf->query, ret, skkbuf_b (buf), &list_num);
+					if (num)
+						break;
+				}
+			}
+			tmp = g_list_nth (buf->candidate_list, buf->candidate_count);
+			buf->candidate_list = g_list_remove_link (buf->candidate_list, tmp);
+			skk_dict_item_destroy ((SkkDictItem *)tmp->data);
+			g_list_free (tmp);
+			/* found */
 			if (num) {
+				buf->candidate_list = skk_dict_item_merge_list (buf->candidate_list, num);
+				buf->candidate_max = g_list_length (buf->candidate_list);
+#if 0
 				for ( tmp = num , count = buf->candidate_count
 						; tmp ;
 						tmp = g_list_next (tmp)) {
-					DEBUG_DO (g_message ("skkbuffer.c : query_num tmp->data %s\n",(gchar *)tmp->data));
+					/* FIXME */
 					buf->candidate_list =
-						g_list_insert (buf->candidate_list, tmp->data, count);
+						g_list_insert (buf->candidate_list, tmp->data , count);
+					g_message ("candidate %s", ((SkkDictItem *)tmp->data)->candidate);
 					count++;
 				}
 				buf->candidate_max += list_num;
 				g_list_free (num);
+				/*
 				query_num (buf);
+				*/
+#endif
 			}
 			g_free (ret);
 		} else {
 			tmp = g_list_nth (buf->candidate_list, buf->candidate_count);
-			g_free (tmp->data);
-			tmp->data = ret;
+			skk_dict_item_destroy ((SkkDictItem *)tmp->data);
+			tmp->data = skk_dict_item_new_with_value (ret, NULL);
+			g_free (ret);
 		}
 	}
 	return;
@@ -377,8 +406,7 @@ query_dict (SkkBuffer *buf)
 	if (preedit) {
 		set_preedit_buf (buf, preedit);
 		g_free (preedit);
-	}
-	else {
+	} else {
 		set_preedit_buf (buf, "");
 		adddict_emit (buf);
 	}
@@ -1356,17 +1384,17 @@ clear (SkkBuffer *buf)
 	if (skkbuf_d (buf))
 		set_direction_word (buf, NULL);
 	if (buf->candidate_list) {
-		skk_utils_list_free (buf->candidate_list,TRUE);
+		skk_dict_item_destroy_all (buf->candidate_list);
 		buf->candidate_list = NULL;
 	}
 	if (buf->number_list) {
-		skk_utils_list_free (buf->number_list,TRUE);
+		skk_utils_list_free (buf->number_list, TRUE, NULL, NULL);
 		buf->number_list = NULL;
 		buf->in_number = FALSE;
 		buf->has_num = 0;
 	}
 	if (buf->completion_list) {
-		skk_utils_list_free (buf->completion_list, TRUE);
+		skk_utils_list_free (buf->completion_list, TRUE, NULL, NULL);
 		buf->completion_list = NULL;
 		buf->completion_count = 0;
 	}
@@ -1553,7 +1581,7 @@ create_candidate (SkkBuffer *buf)
 
 	/* if we already have a candidate_list, free it */
 	if (buf->candidate_list) {
-		skk_utils_list_free (buf->candidate_list, TRUE);
+		skk_dict_item_destroy_all (buf->candidate_list);
 		buf->candidate_list = NULL;
 	}
 	/* if we have a hankan_okuri_char, use it (SKK_OKURI -> SKK_CHOICE) */
@@ -1624,8 +1652,8 @@ create_candidate (SkkBuffer *buf)
 	}
 
 	/* return string */
-	if (buf->candidate_list)
-		return g_strdup ((gchar*)buf->candidate_list->data);
+	if (buf->candidate_list) 
+		return g_strdup (((SkkDictItem *)buf->candidate_list->data)->candidate);
 	else
 		return NULL;
 }
@@ -1666,10 +1694,7 @@ skk_buffer_new (void)
 	memset (ret,0,sizeof (SkkBuffer));
 	ret->mode = skk_mode_new ();
 	ret->conf = NULL;
-	ret->query = skk_query_new ();
-	/* TODO configurable */
-	skk_query_add_item (ret->query, skk_query_item_new (NULL, SKKQUERY_LOCAL));
-	skk_query_add_item (ret->query, skk_query_item_new (NULL, SKKQUERY_SERVER));
+	ret->query = NULL;
 	/* set default functions. */
 	ret->clear = clear;
 	ret->set_status = set_status;
@@ -1700,23 +1725,23 @@ skk_buffer_destroy (SkkBuffer *buf)
 	skk_mode_destroy (buf->mode);
 	skk_conf_destroy (buf->conf);
 	if (buf->preedit_listener) {
-		skk_utils_list_free (buf->preedit_listener, TRUE);
+		skk_utils_list_free (buf->preedit_listener, TRUE, NULL, NULL);
 		buf->preedit_listener = NULL;
 	}
 	if (buf->result_listener) {
-		skk_utils_list_free (buf->result_listener, TRUE);
+		skk_utils_list_free (buf->result_listener, TRUE, NULL, NULL);
 		buf->result_listener = NULL;
 	}
 	if (buf->lookup_listener) {
-		skk_utils_list_free (buf->lookup_listener, TRUE);
+		skk_utils_list_free (buf->lookup_listener, TRUE, NULL, NULL);
 		buf->lookup_listener = NULL;
 	}
 	if (buf->adddict_listener) {
-		skk_utils_list_free (buf->adddict_listener, TRUE);
+		skk_utils_list_free (buf->adddict_listener, TRUE, NULL, NULL);
 		buf->adddict_listener = NULL;
 	}
 	if (buf->codetable_listener) {
-		skk_utils_list_free (buf->adddict_listener, TRUE);
+		skk_utils_list_free (buf->adddict_listener, TRUE, NULL, NULL);
 		buf->codetable_listener = NULL;
 	}
 	g_free (buf);
@@ -1727,7 +1752,8 @@ gboolean
 skk_buffer_add_char (SkkBuffer *buf, int ch)
 {
 	SkkStatus status;
-	if (!buf) return FALSE;
+	if (!buf) 
+		return FALSE;
 	status = skk_buffer_get_status (buf);	
 
 	/* FIXME */
@@ -1788,7 +1814,8 @@ void
 skk_buffer_set_query_status (SkkBuffer *buf, SkkQueryStatus status)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_query_status\n"));
-	if (!buf) return;
+	if (!buf) 
+		return;
 	if (buf->set_query_status) {
 		buf->set_query_status (buf,status);
 	}
@@ -1799,7 +1826,8 @@ SkkQueryStatus
 skk_buffer_get_query_status (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_query_status\n"));
-	if (!buf) return QUERY_NONE;
+	if (!buf) 
+		return QUERY_NONE;
 	return skk_mode_get_query_status (buf->mode);
 }
 
@@ -1807,7 +1835,8 @@ void
 skk_buffer_set_status (SkkBuffer *buf, SkkStatus status)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_status\n"));
-	if (!buf) return;
+	if (!buf)
+		return;
 	if (buf->set_status) {
 		buf->set_status (buf,status);
 	}
@@ -1817,7 +1846,8 @@ SkkStatus
 skk_buffer_get_status (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_status\n"));
-	if (!buf) return SKK_J_MODE;
+	if (!buf) 
+		return SKK_J_MODE;
 	if (buf->get_status) {
 		return buf->get_status (buf);
 	} else {
@@ -1829,7 +1859,8 @@ void
 skk_buffer_set_j_status (SkkBuffer *buf, SkkJStatus jstatus)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_j_status\n"));
-	if (!buf) return;
+	if (!buf) 
+		return;
 	if (buf->set_j_status) {
 		buf->set_j_status (buf,jstatus);
 	}
@@ -1839,7 +1870,8 @@ SkkJStatus
 skk_buffer_get_j_status (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_j_status\n"));
-	if (!buf) return SKK_NONE;
+	if (!buf) 
+		return SKK_NONE;
 	if (buf->get_j_status) {
 		return buf->get_j_status (buf);
 	} else {
@@ -1851,23 +1883,13 @@ void
 skk_buffer_clear (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_clear\n"));
-	if (!buf) return;
+	if (!buf) 
+		return;
 	if (buf->clear) {
 		buf->clear (buf);
 	}
 }
 
-void
-skk_buffer_query_server (SkkBuffer *buf) 
-{
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_query_server\n"));
-	if (!buf) return;
-	skk_mode_prepare_mark (buf->mode);
-	skkbuf_p (buf) = create_candidate (buf);
-	skk_mode_set_j_status (buf->mode, SKK_CHOICE);
-	return;
-}
-
 const gchar *
 skk_buffer_get_first_candidate (SkkBuffer *buf)
 {
@@ -1898,7 +1920,7 @@ skk_buffer_has_next_candidate (SkkBuffer *buf)
 		while (skk_query_set_next (buf->query)) {
 			append_list = skk_query_do_query (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), NULL);
 			if (append_list) {
-				buf->candidate_list = skk_utils_list_merge_string (buf->candidate_list, append_list);
+				buf->candidate_list = skk_dict_item_merge_list (buf->candidate_list, append_list);
 				buf->candidate_max = g_list_length (buf->candidate_list);
 				break;
 			}
@@ -1922,50 +1944,21 @@ skk_buffer_set_next_candidate (SkkBuffer *buf)
 	if (!buf)
 		return FALSE;
 	if (!buf->candidate_list) {
-		GList *append_list = NULL;
-		while (skk_query_set_next (buf->query)) {
-			append_list = skk_query_do_query (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), NULL);
-			if (append_list) {
-				buf->candidate_list = skk_utils_list_merge_string (buf->candidate_list, append_list);
-				buf->candidate_max = g_list_length (buf->candidate_list);
-				/* DEBUG */
-				{
-					GList *tmp_list;
-					for (tmp_list = append_list; tmp_list; tmp_list = g_list_next (tmp_list)) {
-						if (tmp_list && tmp_list->data) 
-							g_message ("tmp_list %s", (gchar *)tmp_list->data);
-					}
-				}	
-				break;
-			}
-		}
-		if (!buf->candidate_list)
+		if (!skk_buffer_has_next_candidate (buf)) {
 			return FALSE;
-	}	
+		}
+	}
 	buf->candidate_count++;
 	if (buf->has_num) {
 		query_num (buf);
 	}
 	candidate = skkbuf_cur_candidate (buf);
-	if (!candidate) {
-		GList *append_list = NULL;
-		while (skk_query_set_next (buf->query)) {
-			append_list = skk_query_do_query (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), NULL);
-			if (append_list) {
-				buf->candidate_list = g_list_append (buf->candidate_list, append_list);
-				buf->candidate_max = g_list_length (buf->candidate_list);
-				break;
-			}
-		}
-		candidate = skkbuf_cur_candidate (buf);
-		if (!candidate)
-			return FALSE;
-	}	
 	if (candidate) {
 		set_preedit_buf (buf,candidate);
 		preedit_emit (buf);
 		return TRUE;
 	}
+	return FALSE;
 }
 
 gboolean
@@ -2025,6 +2018,8 @@ skk_buffer_get_cur_candidate (SkkBuffer *buf)
 		return NULL;
 	if (!buf->candidate_list)
 		return NULL;
+	if (buf->candidate_count == buf->candidate_max)
+		return NULL;
 	if (buf->has_num) {
 		query_num (buf);
 	}
@@ -2065,9 +2060,12 @@ gchar *
 skk_buffer_get_prev_candidate (SkkBuffer *buf) {
 	gchar *candidate;
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_prev_candidate\n"));
-	if (!buf) return NULL;
-	if (!buf->candidate_list) return NULL;
-	if (buf->candidate_count < 1) return NULL;
+	if (!buf) 
+		return NULL;
+	if (!buf->candidate_list) 
+		return NULL;
+	if (buf->candidate_count < 1) 
+		return NULL;
 	buf->candidate_count--;
 	candidate = skkbuf_cur_candidate (buf);
 	if (!candidate)
@@ -2083,7 +2081,8 @@ gchar*
 skk_buffer_get_preedit_string (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_preedit_string\n"));
-	if (!buf) return NULL;
+	if (!buf) 
+		return NULL;
 	if (buf->get_preedit_string) {
 		return buf->get_preedit_string (buf);
 	} else {
@@ -2095,7 +2094,9 @@ const gchar*
 skk_buffer_get_status_string (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_status_string\n"));
-	if (!buf) return g_strdup("Unknown");
+	/* FIXME OK ? */
+	if (!buf) 
+		return g_strdup("Unknown");
 	if (buf->get_status_string) {
 		return buf->get_status_string (buf);
 	} else {
@@ -2107,7 +2108,8 @@ gboolean
 skk_buffer_get_katakana (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_katakana\n"));
-	if (!buf) return FALSE;
+	if (!buf) 
+		return FALSE;
 	return skk_mode_get_katakana (buf->mode);
 }
 
@@ -2127,7 +2129,8 @@ void
 skk_buffer_delete_backward (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_delete_backward\n"));
-	if (!buf) return;
+	if (!buf) 
+		return;
 	if (buf->delete_backward) {
 		buf->delete_backward (buf);
 	}
@@ -2146,7 +2149,8 @@ gboolean
 skk_buffer_can_commit (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_can_commit\n"));
-	if (!buf) return FALSE;
+	if (!buf) 
+		return FALSE;
 	if (buf->can_commit) {
 		return buf->can_commit (buf);
 	}
@@ -2165,7 +2169,8 @@ gboolean
 skk_buffer_has_preedit (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_has_preedit\n"));
-	if (!buf) return FALSE;
+	if (!buf) 
+		return FALSE;
 	if (buf->has_preedit) {
 		return buf->has_preedit (buf);
 	}
@@ -2202,7 +2207,8 @@ gboolean
 skk_buffer_can_clear (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_can_clear\n"));
-	if (!buf) return FALSE;
+	if (!buf)
+		return FALSE;
 	if (buf->can_clear) {
 		return buf->can_clear (buf);
 	}
@@ -2281,13 +2287,27 @@ skk_buffer_commit (SkkBuffer *buf, gboolean regist_dict)
 	return;
 }
 
+void
+skk_buffer_set_query (SkkBuffer *buf, SkkQuery *query)
+{
+	if (!buf)
+		return;
+	if (buf->query) {
+		skk_query_destroy (buf->query);
+		buf->query = NULL;
+	}
+	buf->query = query;
+	skk_query_ref (buf->query);
+	return;
+}
+
 void
 skk_buffer_set_conf (SkkBuffer *buf, SkkConf *conf)
 {
 	if (!buf)
 		return;
 	if (buf->conf) {
-		skk_conf_destroy (conf);
+		skk_conf_destroy (buf->conf);
 		buf->conf = NULL;
 	}
 	buf->conf = conf;
@@ -2399,7 +2419,8 @@ skk_buffer_add_dict (SkkBuffer *buf)
 		return;
 	if (!skkbuf_d (buf))
 		return;
-	skk_ldict_add_word (skkbuf_d (buf), skkbuf_okurip (buf), skkbuf_r (buf));
+	skk_query_set_with_type (buf->query, SKKQUERY_LOCAL);
+	skk_query_add (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), skkbuf_r (buf));
 	return;
 }
 
@@ -2412,7 +2433,8 @@ skk_buffer_add_dict_with_value (SkkBuffer *buf, gchar *value)
 		return;
 	if (!skkbuf_d (buf))
 		return;
-	skk_ldict_add_word (skkbuf_d (buf), skkbuf_okurip (buf), value);
+	skk_query_set_with_type (buf->query, SKKQUERY_LOCAL);
+	skk_query_add (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), value);
 	return;
 }
 
@@ -2420,16 +2442,28 @@ void
 skk_buffer_get_completion (SkkBuffer *buf)
 {
 	gchar *value;
+	GList *append = NULL;
 	if (!buf)
 		return;
 	if (buf->completion_list) {
-		skk_utils_list_free (buf->completion_list, TRUE);
+		skk_utils_list_free (buf->completion_list, TRUE, NULL, NULL);
 		buf->completion_count = 0;
 		buf->completion_list = NULL;
 	}
 	if (!skkbuf_d (buf))
 		return;
-	buf->completion_list = skk_ldict_get_completion (skkbuf_d (buf));
+	/* initialized before completion */
+	skk_query_set_first (buf->query);
+	buf->completion_list = skk_query_do_completion (buf->query, skkbuf_d (buf));
+	while (skk_query_set_next (buf->query)) {
+		append = skk_query_do_completion (buf->query, skkbuf_d (buf));
+		if (append)
+			buf->completion_list = skk_utils_list_merge_string (buf->completion_list, append);
+	}
+	/* re-initialized */
+	skk_query_set_first (buf->query);
+	if (!buf->completion_list)
+		return;
 	value = (gchar *)g_list_nth_data (buf->completion_list, 0);
 	if (!value)
 		return;
