@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkbuffer.h,v 1.4 2002/04/11 00:02:36 famao Exp $ */
+/* $Id: skkbuffer.h,v 1.4.2.1 2002/04/15 21:29:56 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -43,6 +43,7 @@
 #include "skkmode.h"
 #include "skkconf.h"
 #include "skkquery.h"
+#include "skkdictitem.h"
 
 /* in_buf */
 #define skkbuf_b(x) (x)->in_buf
@@ -64,11 +65,14 @@
 
 /* candidate */
 #define skkbuf_candidate_num(x)		(x)->candidate_count
-#define skkbuf_last_candidate(x) (gchar *)((g_list_last ((x)->candidate_list))->data)
-#define skkbuf_first_candidate(x) (gchar *)((g_list_first ((x)->candidate_list))->data)
+#define skkbuf_last_candidate(x) (gchar *)(((SkkDictItem*)g_list_last ((x)->candidate_list)->data)->candidate)
+#define skkbuf_last_annotate(x)	(gchar *)(((SkkDictItem*)g_list_last ((x)->candidate_list)->data)->annotate)
+#define skkbuf_first_candidate(x) (gchar *)(((SkkDictItem*)g_list_first ((x)->candidate_list)->data)->candidate)
+#define skkbuf_first_annotate(x) (gchar *)(((SkkDictItem*)g_list_first ((x)->candidate_list)->data)->annotation)
 #define skkbuf_nth_candidate(x,y)  (gchar *)((g_list_nth_data ((x)->candidate_list,(y))))
 #define skkbuf_cur_candidate_list(x) (g_list_nth ((x)->candidate_list, (x)->candidate_count))
-#define skkbuf_cur_candidate(x)		(gchar *)((g_list_nth_data ((x)->candidate_list, (x)->candidate_count)))
+#define skkbuf_cur_candidate(x)		(gchar *)(((SkkDictItem*)(g_list_nth_data ((x)->candidate_list, (x)->candidate_count)))->candidate)
+#define skkbuf_cur_annotate(x)		(gchar *)(((SkkDictItem*)(g_list_nth_data ((x)->candidate_list, (x)->candidate_count)))->annotate)
 #define skkbuf_last_num_list(x) (gchar *)((g_list_last ((x)->number_list))->data)
 #define skkbuf_first_num_list(x) (gchar *)((g_list_first ((x)->number_list))->data)
 
@@ -240,6 +244,9 @@ void skk_buffer_commit (SkkBuffer *buf, gboolean regist_dict);
 /* set configuration */
 void skk_buffer_set_conf (SkkBuffer *buf, SkkConf *conf);
 
+/* set query */
+void skk_buffer_set_query (SkkBuffer *buf, SkkQuery *query);
+
 /* Event Listener */
 void skk_buffer_add_preedit_listener (SkkBuffer *buf, SkkBufListener listener, gpointer user_data);
 void skk_buffer_add_result_listener (SkkBuffer *buf, SkkBufListener listener, gpointer user_data);
