@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkconv_kana.c,v 1.1.1.1 2002/04/06 02:23:30 famao Exp $ */
+/* $Id: skkconv_kana.c,v 1.1.1.1.2.3 2002/04/15 00:07:30 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -51,36 +51,6 @@ G_LOCK_DEFINE_STATIC(search);
 G_LOCK_DEFINE_STATIC(conv_kana);
 G_LOCK_DEFINE_STATIC(conv_kata);
 
-static gboolean
-is_hiragana (const gchar *str)
-{
-	gint len;
-	if (!str)
-		return FALSE;
-	len = strlen (str);
-	if (len < 2)
-		return FALSE;
-	if ( ((str[0] & 0xff) == 0xa4)) { /* dirty check */
-		return TRUE;
-	}
-	return FALSE;
-}
-
-static gboolean
-is_katakana (const gchar *str)
-{
-	gint len;
-	if (!str)
-		return FALSE;
-	len = strlen (str);
-	if (len < 2)
-		return FALSE;
-	if (((str[0] & 0xff) == 0xa5)) {
-		return TRUE;
-	}
-	return FALSE;
-}
-
 static base_rule_list*
 do_search (const gchar *key)
 {
@@ -293,16 +263,32 @@ skkconv_get_type (const gchar *key)
 gchar*
 skkconv_hiragana_to_katakana (const gchar *hiragana)
 {
-	gchar *copy,*copy2;
+	gchar *ret, *buf;
+	gint len;
+	gint actual_len = 0;
 	if (!hiragana)
 		return NULL;
-	copy = copy2 = g_strdup (hiragana);
-	for (; copy2 && *copy2 ; copy2 += skk_utils_charbytes (copy2)) {
-		if (is_hiragana (copy2)) {
-			*copy2 += 0x01;
+	ret = buf = g_new (gchar, strlen (hiragana) + 1);
+	while (hiragana && *hiragana) {
+		len = skk_utils_charbytes (hiragana);
+		if (skk_utils_is_hiragana (hiragana)) {
+			if (!strncmp (hiragana, "", 4)) {
+				strcpy (buf, "");
+				hiragana += 2;
+			} else {
+				buf[0] = hiragana[0] + 0x01;
+				buf[1] = hiragana[1];
+			}
+		} else {
+			strncpy (buf, hiragana, len);
 		}
+		hiragana += len;
+		buf += len;
+		actual_len += len;
 	}
-	return copy;
+	ret[actual_len] = '\0';
+	ret = g_realloc (ret, actual_len);
+	return ret;
 }
 
 /**
@@ -316,18 +302,41 @@ skkconv_hiragana_to_katakana (const gchar *hiragana)
 gchar*
 skkconv_katakana_to_hiragana (const gchar *katakana)
 {
-	/* TODO */
+	/* NOTICE */
 	/* katakana has more characters than hiragana */
-	gchar *copy,*copy2;
+	gchar *ret, *buf;
+	gint len;
+	gint actual_len = 0;
 	if (!katakana)
 		return NULL;
-	copy = copy2 = g_strdup (katakana);
-	for (; copy2 && *copy2; copy2 += skk_utils_charbytes (copy2)) {
-		if (is_katakana (copy2)) {
-			*copy2 -= 0x01;
+	/* In the case of ֥, we need strlen * 2 */
+	ret = buf = g_new (gchar, (strlen (katakana) * 2) + 1);
+	while (katakana && *katakana) {
+		len = skk_utils_charbytes (katakana);
+		if (skk_utils_is_katakana (katakana)) {
+			if ((katakana[1] & 0xff) == 0xf4) { /*  */
+				strcpy (buf,"");
+				/* increment only 2. not 4 */
+				buf += 2;
+				actual_len += 2;
+			} else if (((katakana[1] & 0xff) == 0xf5) ||
+					((katakana[1] & 0xff) == 0xf6)) { /*  */
+				/* there is no hiragana, not translate */
+				strncpy (buf, katakana, len);
+			} else {
+				buf[0] = katakana[0] - 0x01;
+				buf[1] = katakana[1];
+			}
+		} else {
+			strncpy (buf, katakana, len);
 		}
+		katakana += len;
+		buf += len;
+		actual_len += len;
 	}
-	return copy;
+	ret[actual_len] = '\0';
+	ret = g_realloc (ret, actual_len);
+	return ret;
 }
 
 #ifdef SKKCONV_KANA_DEBUG
@@ -350,7 +359,10 @@ main (void)
 	printf ("%d\n",skkconv_is_exist ("fuga"));
 	printf ("%d\n",skkconv_is_exist ("fuga"));
 	printf ("%s\n",skkconv_hiragana_to_katakana ("礦1󤭤Ǥ"));
-	printf ("%s\n",skkconv_katakana_to_hiragana ("1󥭥ǥ"));
+	printf ("%s\n",skkconv_hiragana_to_katakana (""));
+	printf ("%s\n",skkconv_katakana_to_hiragana ("1ǥ"));
+	printf ("%s\n",skkconv_katakana_to_hiragana (""));
+	printf ("%s\n",skkconv_katakana_to_hiragana (""));
 	return 0;
 }
 #endif
