@@ -258,15 +258,15 @@ skk_mode_destroy (SkkMode *mode)
 	if (!mode) return;
 	skk_mode_clear (mode);
 	if (mode->status_listener) {
-		skk_utils_list_free (mode->status_listener, TRUE);
+		skk_utils_list_free (mode->status_listener, TRUE, NULL, NULL);
 		mode->status_listener = NULL;
 	}
 	if (mode->j_status_listener) {
-		skk_utils_list_free (mode->j_status_listener, TRUE);
+		skk_utils_list_free (mode->j_status_listener, TRUE, NULL, NULL);
 		mode->j_status_listener = NULL;
 	}
 	if (mode->query_status_listener) {
-		skk_utils_list_free (mode->query_status_listener, TRUE);
+		skk_utils_list_free (mode->query_status_listener, TRUE, NULL, NULL);
 		mode->query_status_listener = NULL;
 	}	
 	g_free (mode);
