@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: skk.c,v 1.9 2002/04/11 00:31:27 famao Exp $ */
+/* $Id: skk.c,v 1.9.2.3 2002/04/24 22:13:01 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -89,7 +89,6 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include "csconv.h"
 #include "skkcomm.h"
 #include "skkbuffer.h"
-#include "skkdict.h"
 #include "skkconv_jisx0208_latin.h"
 #include "skkkeymap.h"
 #include "skkkeysym.h"
@@ -111,20 +110,20 @@ Sun Microsystems, Inc. or its licensors is granted.
 /*
  * define If method prototype
  */
-Bool if_skk_OpenIF (iml_if_t *);
-Bool if_skk_CloseIF (iml_if_t *);
-Bool if_skk_GetIFValues (iml_if_t *, IMArgList, int);
-Bool if_skk_SetIFValues (iml_if_t *, IMArgList, int);
-Bool if_skk_OpenDesktop (iml_desktop_t *, IMArgList, int);
-Bool if_skk_CloseDesktop (iml_desktop_t *);
-Bool if_skk_CreateSC (iml_session_t *, IMArgList, int);
-Bool if_skk_DestroySC (iml_session_t *);
-Bool if_skk_GetSCValues (iml_session_t *, IMArgList, int);
-Bool if_skk_SetSCValues (iml_session_t *, IMArgList, int);
-IMText *if_skk_ResetSC (iml_session_t *);
-void if_skk_SetSCFocus (iml_session_t *);
-void if_skk_UnsetSCFocus (iml_session_t *);
-void if_skk_SendEvent (iml_session_t *, IMInputEvent * ev);
+static Bool if_skk_OpenIF (iml_if_t *);
+static Bool if_skk_CloseIF (iml_if_t *);
+static Bool if_skk_GetIFValues (iml_if_t *, IMArgList, int);
+static Bool if_skk_SetIFValues (iml_if_t *, IMArgList, int);
+static Bool if_skk_OpenDesktop (iml_desktop_t *, IMArgList, int);
+static Bool if_skk_CloseDesktop (iml_desktop_t *);
+static Bool if_skk_CreateSC (iml_session_t *, IMArgList, int);
+static Bool if_skk_DestroySC (iml_session_t *);
+static Bool if_skk_GetSCValues (iml_session_t *, IMArgList, int);
+static Bool if_skk_SetSCValues (iml_session_t *, IMArgList, int);
+static IMText *if_skk_ResetSC (iml_session_t *);
+static void if_skk_SetSCFocus (iml_session_t *);
+static void if_skk_UnsetSCFocus (iml_session_t *);
+static void if_skk_SendEvent (iml_session_t *, IMInputEvent * ev);
 
 /*
  * define if_methods_t
@@ -195,7 +194,7 @@ static void skk_lookup_choice_done (iml_session_t *s);
 static void skk_aux_start (iml_session_t *s, char *class_name);
 static void skk_aux_status_start (iml_session_t *s);
 static void skk_aux_dict_start (iml_session_t *s);
-static void skk_aux_dict_draw (iml_session_t *s);
+static void skk_aux_dict_draw (iml_session_t *s, const gchar *query_str);
 
 static void skk_start_conversion (iml_session_t *s);
 static void skk_end_conversion (iml_session_t *s);
@@ -214,7 +213,7 @@ static void skk_aux_status_draw_off (iml_session_t *s);
 static IMText *skk_utftoimtext (iml_session_t * s, UTFCHAR * p);
 static IMText *skk_strtoimtext (iml_session_t * s, int nseg, int *nb,
 			 unsigned char **strs, int *feedback_type);
-static UTFCHAR *skk_strtoutf (unsigned char *str);
+static UTFCHAR *skk_strtoutf (const unsigned char *str);
 
 
 typedef enum
@@ -226,9 +225,11 @@ typedef enum
 SKK_Feedback_Types;
 
 static int skk_context_id_counter = 1;
-static int skk_cur_sock = -1;
 
-typedef struct
+typedef struct _SkkDesktopData SkkDesktopData;
+typedef struct _SkkSessionData SkkSessionData;
+
+struct _SkkSessionData
 {
 	int context_id;
 	int conversion_start;
@@ -241,33 +242,42 @@ typedef struct
 	int dict_aux_ok;
 	int palette_aux_ok;
 	SkkBuffer *skkbuf;
+};
+
+struct _SkkDesktopData
+{
 	SkkFunc *skkfunc;
-}
-SkkLESession;
+	SkkQuery *skkquery;
+	SkkConf *skkconf;
+};
 
 
-SkkLESession *
+static SkkSessionData *skk_session_data (iml_session_t *s);
+static SkkDesktopData *skk_desktop_data (iml_session_t *s);
+static SkkBuffer *skk_session_buf (iml_session_t *s);
+
+static SkkSessionData *
 skk_session_data (iml_session_t * s)
 {
-  return (SkkLESession *) (s->specific_data);
+	return (SkkSessionData *) (s->specific_data);
 }
 
-SkkBuffer *
+static SkkDesktopData *
+skk_desktop_data (iml_session_t *s)
+{
+	return (SkkDesktopData *) (s->desktop->specific_data);
+}
+
+static SkkBuffer *
 skk_session_buf (iml_session_t * s)
 {
-	return (SkkBuffer *) (((SkkLESession *) (s->specific_data))->skkbuf);
+	return (SkkBuffer *) (((SkkSessionData *) (s->specific_data))->skkbuf);
 }
 
-SkkFunc *
-skk_session_func (iml_session_t * s)
-{
-	return (SkkFunc *) (((SkkLESession *) (s->specific_data))->skkfunc);
-}
-
-int
+static int
 skk_session_context (iml_session_t * s)
 {
-  return ((SkkLESession *) (s->specific_data))->context_id;
+	return ((SkkSessionData *) (s->specific_data))->context_id;
 }
 
 /*
@@ -449,7 +459,7 @@ if_GetIfInfo (IMArgList args, int num_args)
 	}
 }
 
-Bool
+static Bool
 if_skk_OpenIF (iml_if_t * If)
 {
 	int st;
@@ -462,13 +472,13 @@ if_skk_OpenIF (iml_if_t * If)
 	return True;
 }
 
-Bool
+static Bool
 if_skk_CloseIF (iml_if_t * If)
 {
 	return True;
 }
 
-Bool
+static Bool
 if_skk_GetIFValues (iml_if_t * If, IMArgList args, int num_args)
 {
 #ifdef DEBUG
@@ -479,7 +489,7 @@ if_skk_GetIFValues (iml_if_t * If, IMArgList args, int num_args)
 	return True;
 }
 
-Bool
+static Bool
 if_skk_SetIFValues (iml_if_t * If, IMArgList args, int num_args)
 {
 #ifdef DEBUG
@@ -490,10 +500,12 @@ if_skk_SetIFValues (iml_if_t * If, IMArgList args, int num_args)
 	return True;
 }
 
-Bool
+static Bool
 if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 {
 	int i;
+	SkkDesktopData *d;
+
 #ifdef DEBUG
 	printf ("if_skk_OpenDesktop()\n");
 	printf ("	This method is invoked when the user start to use\n");
@@ -556,15 +568,22 @@ if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 				break;
 		}
 	}
-	if (skk_cur_sock < 0) {
-		skk_cur_sock = skk_comm_makesock ("localhost", "skkserv");
-	}
+	d = g_new0 (SkkDesktopData, 1);
+	d->skkfunc = skk_func_new ();
+	d->skkquery = skk_query_new ();
+	d->skkconf = skk_conf_new ();
+	/* TODO configurable */
+	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_LOCAL));
+	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_SERVER));
+	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_LOOK));
+	desktop->specific_data = (void *)d;
 	return True;
 }
 
-Bool
+static Bool
 if_skk_CloseDesktop (iml_desktop_t * desktop)
 {
+	SkkDesktopData *d = (SkkDesktopData *)desktop->specific_data;
 #ifdef DEBUG
 	printf ("if_skk_CloseDesktop()\n");
 	printf ("	This method is invoked when the user exits from\n");
@@ -575,16 +594,28 @@ if_skk_CloseDesktop (iml_desktop_t * desktop)
 	printf ("	user [%s@%s] exited\n",
 			desktop->user_name, desktop->host_name);
 	printf ("\n");
-#endif 
-	if (skk_cur_sock > 0) {
-		skk_comm_bye (skk_cur_sock);
+#endif
+#if 0
 		shutdown (skk_cur_sock, 2);
+		skk_comm_bye (skk_cur_sock);
+		skk_comm_bye (skk_cur_sock);
 		skk_cur_sock = -1;
 	}
+#endif
 	/*
 	skk_aux_status_done (s);
 	skk_aux_dict_done (s);
 	*/
+	if (d->skkquery) {
+		skk_query_destroy (d->skkquery);
+	}
+	if (d->skkfunc) {
+		skk_func_destroy (d->skkfunc);
+	}
+	if (d->skkconf) {
+		skk_conf_destroy (d->skkconf);
+	}
+	g_free (d);
 	return True;
 }
 
@@ -672,7 +703,7 @@ static void
 skk_adddict_listener (SkkBuffer *buf, gchar *query_str, gpointer user_data)
 {
 	iml_session_t *s = (iml_session_t *) user_data;
-	skk_aux_dict_draw (s);
+	skk_aux_dict_draw (s, query_str);
 	return;
 }
 
@@ -689,9 +720,9 @@ skk_codetable_listener (SkkBuffer *buf, gpointer user_data)
 Bool
 if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 {
-	SkkLESession *ses = (SkkLESession *) malloc (sizeof (SkkLESession));
+	SkkDesktopData *d = skk_desktop_data (s);
+	SkkSessionData *ses = g_new0 (SkkSessionData, 1);
 	SkkBuffer *buf = skk_buffer_new ();
-	skk_dict_set_server_socket (skk_cur_sock);
 	skk_buffer_set_status (buf,SKK_LATIN_MODE);
 	if (!ses)
 		return False;
@@ -706,8 +737,9 @@ if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 	ses->luc_hasmore = True;
 	ses->context_id = skk_context_id_counter++;
 	ses->skkbuf = buf;
-	ses->skkfunc = skk_func_new ();
 	s->specific_data = (void *) ses;
+	skk_buffer_set_query (buf, d->skkquery);
+	skk_buffer_set_conf (buf, d->skkconf);
 	skk_buffer_add_result_listener (buf, skk_result_listener, (gpointer) s);
 	skk_buffer_add_preedit_listener (buf, skk_preedit_listener, (gpointer) s);
 	skk_mode_add_status_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
@@ -721,11 +753,11 @@ if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 Bool
 if_skk_DestroySC (iml_session_t * s)
 {
-	SkkLESession *ses;
+	SkkSessionData *ses;
 	ses = skk_session_data (s);
 	skk_buffer_destroy (ses->skkbuf);
 	g_free (ses);
-	return TRUE;
+	return True;
 }
 
 Bool
@@ -862,7 +894,7 @@ if_skk_SendEvent (iml_session_t *s, IMInputEvent *ev)
 static void
 skk_make_conversion_on (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
 	if (ses->conversion_start == False) {
 		ses->conversion_start = True;
@@ -876,7 +908,7 @@ skk_make_conversion_on (iml_session_t *s)
 static void
 skk_make_conversion_off (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->conversion_start == True) {
 		ses->conversion_start = False;
 		skk_status_draw (s);
@@ -890,7 +922,7 @@ static void
 skk_commit_imtext (iml_session_t *s, IMText *text)
 {
 	iml_inst *lp;
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (!text)
 		return;
 
@@ -949,7 +981,7 @@ static int
 skk_translate_keyevent (iml_session_t *s, IMKeyListEvent * kev)
 {
   SkkBuffer *buf = skk_session_buf (s);
-  SkkLESession *ses = skk_session_data (s);
+  SkkSessionData *ses = skk_session_data (s);
 #if 0
   IMKeyEventStruct *k = (IMKeyEventStruct *) kev->keylist;
   printf ("iml_session_t() keycode=%x,keychar=%x,state=%x\n",
@@ -1253,9 +1285,9 @@ skk_process_keyevent (iml_session_t * s, IMKeyListEvent * kev)
 {
 	int ch;
 	int status;
-	gint skk_status = 0;
+	guint skk_status = 0;
+	SkkDesktopData *d = skk_desktop_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
-	SkkFunc *func = skk_session_func (s);
 	IMKeyEventStruct *k = (IMKeyEventStruct *)kev->keylist;
 	ch = skk_translate_keyevent (s, kev);
 	if (ch == SKK_NOTRANS) {
@@ -1277,7 +1309,7 @@ skk_process_keyevent (iml_session_t * s, IMKeyListEvent * kev)
 	if (!skk_status) {
 		skk_status = SKK_NONE_MASK;
 	}
-	status = skk_keymap_do_func (buf, func, ch, skk_status);
+	status = skk_keymap_do_func (buf, d->skkfunc, ch, skk_status);
 	switch (status) {
 		case SKK_NOTHING:
 			return;
@@ -1502,7 +1534,7 @@ skk_utflen (int skk_str_len)
 
 /* Convert String to UTFCHAR */
 static UTFCHAR*
-skk_strtoutf (unsigned char *str)
+skk_strtoutf (const unsigned char *str)
 {
 	int ret, strlength, ustrlength;
 	UTFCHAR *p,*ustr = NULL;
@@ -1614,7 +1646,7 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 {
 	iml_inst *lp;
 	IMText *p;
-	SkkLESession *sses = skk_session_data (s);
+	SkkSessionData *sses = skk_session_data (s);
 
 	if (sses->preedit_start == False) {
 		lp = s->If->m->iml_make_preedit_start_inst (s);
@@ -1640,7 +1672,7 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 static void
 skk_preedit_done (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->preedit_start == True) {
 		iml_inst *lp;
 		lp = s->If->m->iml_make_preedit_done_inst (s);
@@ -1652,7 +1684,7 @@ skk_preedit_done (iml_session_t *s)
 static void
 skk_preedit_clear (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->preedit_start == True) {
 		iml_inst *lp;
 		lp = s->If->m->iml_make_preedit_erase_inst (s);
@@ -1669,7 +1701,7 @@ skk_aux_status_draw_off (iml_session_t *s)
 {
 	int send_status[1];
 	UTFCHAR *send_string[1];
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->palette_aux_ok == False) {
 		skk_aux_start (s, SKK_AUX_PALETTE_CLASS_NAME);
 		/* Send Initialize Request */
@@ -1692,7 +1724,7 @@ skk_aux_status_draw (iml_session_t *s)
 	gchar *str;
 	UTFCHAR *ustr;
 	UTFCHAR *send_string[1];
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
 	SkkStatus status = skk_buffer_get_status (buf);
 
@@ -1744,7 +1776,7 @@ skk_status_draw_off (iml_session_t *s)
 	iml_inst *rrv = NULL;
 	IMText *p;
 	UTFCHAR *ustr;
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	ustr = skk_strtoutf ("SKK(OFF)");
 	p = skk_utftoimtext (s, ustr);
 	if (ses->status_start == False) {
@@ -1769,7 +1801,7 @@ skk_status_draw (iml_session_t *s)
 	IMText *p;
 	UTFCHAR *ustr;
 	gchar *str;
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
 
 	if (ses->conversion_start == False) {
@@ -1797,7 +1829,7 @@ skk_status_draw (iml_session_t *s)
 static void
 skk_start_lookup_choice (iml_session_t *s, iml_inst **prrv, int num)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->luc_start == False) {
 		iml_inst *lp;
 		IMLookupStartCallbackStruct *start;
@@ -1841,7 +1873,7 @@ skk_show_lookup_choice (iml_session_t *s)
 			/* no more choice */
 			skk_lookup_choice_done (s);
 			/* show AddDictAux */
-			skk_aux_dict_draw (s);
+			skk_buffer_adddict_emit (buf);
 			return;
 		}
 	}
@@ -1904,7 +1936,7 @@ skk_show_lookup_choice (iml_session_t *s)
 static void
 skk_lookup_choice_done (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	if (ses->luc_start == True) {
 		iml_inst *lp;
 		lp = s->If->m->iml_make_lookup_done_inst (s);
@@ -1995,7 +2027,7 @@ static void
 skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 {
 	int i;
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
 	for (i = 0; i < aux->count_integer_values; i++) {
 		switch (aux->integer_values[i]) {
@@ -2077,12 +2109,13 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 }
 
 static void
-skk_aux_dict_draw (iml_session_t *s)
+skk_aux_dict_draw (iml_session_t *s, const gchar *query_str)
 {
-	SkkLESession *ses = skk_session_data (s);
-	SkkBuffer *buf = skk_session_buf (s);
+	SkkSessionData *ses = skk_session_data (s);
 	int status[1];
 	UTFCHAR *send_string[1];
+	if (!query_str)
+		return;
 	if (ses->dict_aux_ok == False) {
 		skk_aux_start (s, SKK_AUX_ADD_DICT_CLASS_NAME);
 		/* send initialize request */
@@ -2093,14 +2126,7 @@ skk_aux_dict_draw (iml_session_t *s)
 	} /* TODO */
 	if (ses->dict_start == False) {
 		status[0] = SKK_AUX_DICT_ADD;
-		if (skkbuf_okuri (buf)) {
-			gchar *direction;
-			direction = g_strdup_printf ("%s*%s", skkbuf_d (buf), skkbuf_okuri (buf));
-			send_string[0] = skk_strtoutf (direction);
-			g_free (direction);
-		} else {
-			send_string[0] = skk_strtoutf (skkbuf_d (buf));
-		}
+		send_string[0] = skk_strtoutf (query_str);
 		skk_aux_draw (s, SKK_AUX_ADD_DICT_CLASS_NAME,
 				1, status, 1, send_string);
 		g_free (send_string[0]);
@@ -2130,7 +2156,7 @@ skk_start_conversion (iml_session_t *s)
 static void
 skk_status_done (iml_session_t *s)
 {
-	SkkLESession *ses = skk_session_data (s);
+	SkkSessionData *ses = skk_session_data (s);
 	iml_inst *lp;
 	if (ses->status_start == True) {
 		lp = s->If->m->iml_make_status_done_inst (s);
